@@ -91,6 +91,8 @@ static gboolean wnck_pager_button_release (GtkWidget        *widget,
                                            GdkEventButton   *event);
 static gboolean wnck_pager_focus         (GtkWidget        *widget,
                                           GtkDirectionType  direction);
+static void workspace_name_changed_callback (WnckWorkspace *workspace,
+                                             gpointer       data);
 
 static void wnck_pager_connect_screen    (WnckPager  *pager,
                                           WnckScreen *screen);
@@ -623,7 +625,8 @@ static void
 wnck_pager_draw_workspace (WnckPager    *pager,
 			   int           workspace,
 			   GdkRectangle *rect,
-                           GdkPixbuf    *bg_pixbuf)
+                           GdkPixbuf    *bg_pixbuf,
+                           gboolean      prelight)
 {
   GList *windows;
   GList *tmp;
@@ -640,9 +643,16 @@ wnck_pager_draw_workspace (WnckPager    *pager,
   
   if (is_current)
     gdk_draw_rectangle (GTK_WIDGET (pager)->window,
+                        prelight ? 
+                        GTK_WIDGET (pager)->style->mid_gc[GTK_STATE_SELECTED] :
                         GTK_WIDGET (pager)->style->dark_gc[GTK_STATE_SELECTED],
                         TRUE,
                         rect->x, rect->y, rect->width, rect->height);
+  else if (prelight)
+    gdk_draw_rectangle (GTK_WIDGET (pager)->window,
+                        GTK_WIDGET (pager)->style->dark_gc[GTK_STATE_PRELIGHT],
+                        TRUE,
+                        rect->x, rect->y, rect->width, rect->height);
   else if (bg_pixbuf)
     {
       gdk_pixbuf_render_to_drawable (bg_pixbuf,
@@ -729,6 +739,7 @@ wnck_pager_expose_event  (GtkWidget      *widget,
   WnckWorkspace *active_space;
   GdkPixbuf *bg_pixbuf;
   gboolean first;
+  int drag_space;
   
   pager = WNCK_PAGER (widget);
 
@@ -736,6 +747,13 @@ wnck_pager_expose_event  (GtkWidget      *widget,
   active_space = wnck_screen_get_active_workspace (pager->priv->screen);
   bg_pixbuf = NULL;
   first = TRUE;
+
+  if (pager->priv->dragging)
+    drag_space = workspace_at_point (pager,
+                                     pager->priv->drag_window_x,
+                                     pager->priv->drag_window_y);
+  else
+    drag_space = -1;
   
   i = 0;
   while (i < n_spaces)
@@ -760,41 +778,35 @@ wnck_pager_expose_event  (GtkWidget      *widget,
               first = FALSE;
             }
           
-	  wnck_pager_draw_workspace (pager, i, &rect, bg_pixbuf);
+	  wnck_pager_draw_workspace (pager, i, &rect, bg_pixbuf,
+                                     drag_space == i);
 	}
  
       ++i;
     }
 
-  /* Draw the drag window */
-  if (pager->priv->dragging)
+  /* Draw the drag window */  
+  if (drag_space >= 0)
     {
       GdkRectangle rect;
       GdkRectangle winrect;
       int dx, dy;
       
-      i = workspace_at_point (pager,
-                              pager->priv->drag_window_x,
-                              pager->priv->drag_window_y);
-
-      if (i >= 0)
-	{
-	  get_workspace_rect (pager, i, &rect);          
-	  get_window_rect (pager->priv->drag_window, &rect, &winrect);
-
-	  dx = (pager->priv->drag_window_x - rect.x) -
-	    pager->priv->drag_start_x_workspace_relative;
-	  dy = (pager->priv->drag_window_y - rect.y) -
-	    pager->priv->drag_start_y_workspace_relative;
-	  
-	  winrect.x += dx;
-	  winrect.y += dy;
-	  
-	  draw_window (widget->window,
-		       widget,
-		       pager->priv->drag_window,
-		       &winrect);
-	}
+      get_workspace_rect (pager, drag_space, &rect);          
+      get_window_rect (pager->priv->drag_window, &rect, &winrect);
+      
+      dx = (pager->priv->drag_window_x - rect.x) -
+        pager->priv->drag_start_x_workspace_relative;
+      dy = (pager->priv->drag_window_y - rect.y) -
+        pager->priv->drag_start_y_workspace_relative;
+      
+      winrect.x += dx;
+      winrect.y += dy;
+      
+      draw_window (widget->window,
+                   widget,
+                   pager->priv->drag_window,
+                   &winrect);
     }
   
   return FALSE;
@@ -984,6 +996,19 @@ wnck_pager_focus (GtkWidget        *widget,
   return GTK_WIDGET_CLASS (parent_class)->focus (widget, direction);
 }
 
+void
+wnck_pager_set_screen (WnckPager  *pager,
+		       WnckScreen *screen)
+{
+  if (pager->priv->screen == screen)
+    return;
+
+  if (pager->priv->screen)
+    wnck_pager_disconnect_screen (pager);
+
+  wnck_pager_connect_screen (pager, screen);
+}
+
 GtkWidget*
 wnck_pager_new (WnckScreen *screen)
 {
@@ -1141,6 +1166,8 @@ workspace_created_callback        (WnckScreen      *screen,
                                    gpointer         data)
 {
   WnckPager *pager = WNCK_PAGER (data);
+  g_signal_connect (space, "name_changed",
+                    G_CALLBACK (workspace_name_changed_callback), pager);
   gtk_widget_queue_resize (GTK_WIDGET (pager));
 }
 
@@ -1150,6 +1177,7 @@ workspace_destroyed_callback      (WnckScreen      *screen,
                                    gpointer         data)
 {
   WnckPager *pager = WNCK_PAGER (data);
+  g_signal_handlers_disconnect_by_func (space, G_CALLBACK (workspace_name_changed_callback), pager);
   gtk_widget_queue_resize (GTK_WIDGET (pager));
 }
 
@@ -1158,7 +1186,7 @@ application_opened_callback       (WnckScreen      *screen,
                                    WnckApplication *app,
                                    gpointer         data)
 {
-  WnckPager *pager = WNCK_PAGER (data);
+  /*   WnckPager *pager = WNCK_PAGER (data); */
 }
 
 static void
@@ -1166,7 +1194,7 @@ application_closed_callback       (WnckScreen      *screen,
                                    WnckApplication *app,
                                    gpointer         data)
 {
-  WnckPager *pager = WNCK_PAGER (data);
+  /*   WnckPager *pager = WNCK_PAGER (data); */
 }
 
 static void
@@ -1227,6 +1255,13 @@ background_changed_callback (WnckWindow *window,
   gtk_widget_queue_draw (GTK_WIDGET (pager));
 }
 
+static void
+workspace_name_changed_callback (WnckWorkspace *space,
+                                 gpointer       data)
+{
+  gtk_widget_queue_resize (GTK_WIDGET (data));
+}
+
 static void
 wnck_pager_connect_screen (WnckPager  *pager,
                            WnckScreen *screen)
@@ -1300,6 +1335,15 @@ wnck_pager_connect_screen (WnckPager  *pager,
   ++i;
   
   g_assert (i == N_SCREEN_CONNECTIONS);
+
+  /* connect to name_changed on each workspace */
+  for (i = 0; i < wnck_screen_get_workspace_count (pager->priv->screen); i++)
+    {
+      WnckWorkspace *space;
+      space = wnck_screen_get_workspace (pager->priv->screen, i);
+      g_signal_connect (space, "name_changed",
+                        G_CALLBACK (workspace_name_changed_callback), pager);
+    }
 }
 
 static void
@@ -1343,6 +1387,13 @@ wnck_pager_disconnect_screen (WnckPager  *pager)
       ++i;
     }
 
+  for (i = 0; i < wnck_screen_get_workspace_count (pager->priv->screen); i++)
+    {
+      WnckWorkspace *space;
+      space = wnck_screen_get_workspace (pager->priv->screen, i);
+      g_signal_handlers_disconnect_by_func (space, G_CALLBACK (workspace_name_changed_callback), pager);
+    }
+
   pager->priv->screen = NULL;
 }
 
